@@ -21,6 +21,8 @@ MAKE_ACCEPT(Builtin)
 MAKE_ACCEPT(Identifier)
 MAKE_ACCEPT(PositionalParameter)
 MAKE_ACCEPT(Call)
+MAKE_ACCEPT(Sizeof)
+MAKE_ACCEPT(Offsetof)
 MAKE_ACCEPT(Map)
 MAKE_ACCEPT(Variable)
 MAKE_ACCEPT(Binop)
@@ -53,6 +55,18 @@ Call::~Call()
   delete vargs;
   vargs = nullptr;
 }
+Sizeof::~Sizeof()
+{
+  if (expr)
+    delete expr;
+  expr = nullptr;
+}
+Offsetof::~Offsetof()
+{
+  if (expr)
+    delete expr;
+  expr = nullptr;
+}
 Map::~Map()
 {
   if (vargs)
@@ -207,7 +221,7 @@ Program::~Program()
   probes = nullptr;
 }
 
-Integer::Integer(long n, location loc) : Expression(loc), n(n)
+Integer::Integer(int64_t n, location loc) : Expression(loc), n(n)
 {
   is_literal = true;
 }
@@ -256,6 +270,25 @@ Call::Call(const std::string &func, ExpressionList *vargs, location loc)
 {
 }
 
+Sizeof::Sizeof(SizedType type, location loc)
+    : Expression(loc), expr(nullptr), argtype(type)
+{
+}
+
+Sizeof::Sizeof(Expression *expr, location loc) : Expression(loc), expr(expr)
+{
+}
+
+Offsetof::Offsetof(SizedType record, std::string &field, location loc)
+    : Expression(loc), record(record), expr(nullptr), field(field)
+{
+}
+
+Offsetof::Offsetof(Expression *expr, std::string &field, location loc)
+    : Expression(loc), expr(expr), field(field)
+{
+}
+
 Map::Map(const std::string &ident, location loc)
     : Expression(loc), ident(ident), vargs(nullptr)
 {
@@ -321,18 +354,10 @@ ArrayAccess::ArrayAccess(Expression *expr, Expression *indexpr, location loc)
 {
 }
 
-
-Cast::Cast(const std::string &type,
-           bool is_pointer,
-           bool is_double_pointer,
-           Expression *expr,
-           location loc)
-    : Expression(loc),
-      cast_type(type),
-      is_pointer(is_pointer),
-      is_double_pointer(is_double_pointer),
-      expr(expr)
+Cast::Cast(SizedType cast_type, Expression *expr, location loc)
+    : Expression(loc), expr(expr)
 {
+  type = cast_type;
 }
 
 
@@ -519,15 +544,14 @@ std::string AttachPoint::name(const std::string &attach_point) const
   return name(target, attach_point);
 }
 
-int AttachPoint::index(const std::string &name) const
+int AttachPoint::index() const
 {
-  if (index_.count(name) == 0) return 0;
-  return index_.at(name);
+  return index_;
 }
 
-void AttachPoint::set_index(const std::string &name, int index)
+void AttachPoint::set_index(int index)
 {
-  index_[name] = index;
+  index_ = index;
 }
 
 std::string Probe::name() const
@@ -578,6 +602,14 @@ Call::Call(const Call &other) : Expression(other)
   func = other.func;
 }
 
+Sizeof::Sizeof(const Sizeof &other) : Expression(other)
+{
+}
+
+Offsetof::Offsetof(const Offsetof &other) : Expression(other)
+{
+}
+
 Binop::Binop(const Binop &other) : Expression(other)
 {
   op = other.op;
@@ -614,9 +646,6 @@ Program::Program(const Program &other) : Node(other)
 
 Cast::Cast(const Cast &other) : Expression(other)
 {
-  cast_type = other.cast_type;
-  is_pointer = other.is_pointer;
-  is_double_pointer = other.is_double_pointer;
 }
 
 Probe::Probe(const Probe &other) : Node(other)
@@ -662,5 +691,13 @@ AssignMapStatement::AssignMapStatement(const AssignMapStatement &other)
   compound = other.compound;
 };
 
+SizedType ident_to_record(const std::string &ident, int pointer_level)
+{
+  SizedType result = CreateRecord(ident, std::weak_ptr<Struct>());
+  for (int i = 0; i < pointer_level; i++)
+    result = CreatePointer(result);
+  return result;
+}
+
 } // namespace ast
 } // namespace bpftrace
