@@ -1,5 +1,6 @@
 #pragma once
 
+#include <cstdint>
 #include <map>
 #include <string>
 #include <vector>
@@ -111,9 +112,9 @@ public:
   DEFINE_ACCEPT
   DEFINE_LEAFCOPY(Integer)
 
-  explicit Integer(long n, location loc);
+  explicit Integer(int64_t n, location loc);
 
-  long n;
+  int64_t n;
 
 private:
   Integer(const Integer &other) = default;
@@ -210,6 +211,41 @@ private:
   Call(const Call &other);
 };
 
+class Sizeof : public Expression
+{
+public:
+  DEFINE_ACCEPT
+  DEFINE_LEAFCOPY(Sizeof)
+
+  Sizeof(SizedType type, location loc);
+  Sizeof(Expression *expr, location loc);
+  ~Sizeof();
+
+  Expression *expr;
+  SizedType argtype;
+
+private:
+  Sizeof(const Sizeof &other);
+};
+
+class Offsetof : public Expression
+{
+public:
+  DEFINE_ACCEPT
+  DEFINE_LEAFCOPY(Offsetof)
+
+  Offsetof(SizedType record, std::string &field, location loc);
+  Offsetof(Expression *expr, std::string &field, location loc);
+  ~Offsetof();
+
+  SizedType record;
+  Expression *expr;
+  std::string field;
+
+private:
+  Offsetof(const Offsetof &other);
+};
+
 class Map : public Expression {
 public:
   DEFINE_ACCEPT
@@ -319,20 +355,9 @@ public:
   DEFINE_LEAFCOPY(Cast)
   DEFINE_ACCEPT
 
-  Cast(const std::string &type,
-       bool is_pointer,
-       bool is_double_pointer,
-       Expression *expr);
-  Cast(const std::string &type,
-       bool is_pointer,
-       bool is_double_pointer,
-       Expression *expr,
-       location loc);
+  Cast(SizedType type, Expression *expr, location loc);
   ~Cast();
 
-  std::string cast_type;
-  bool is_pointer;
-  bool is_double_pointer;
   Expression *expr = nullptr;
 
 private:
@@ -551,13 +576,13 @@ public:
   std::string name(const std::string &attach_target,
                    const std::string &attach_point) const;
 
-  int index(const std::string &name) const;
-  void set_index(const std::string &name, int index);
+  int index() const;
+  void set_index(int index);
 
 private:
   AttachPoint(const AttachPoint &other) = default;
 
-  std::map<std::string, int> index_;
+  int index_ = 0;
 };
 using AttachPointList = std::vector<AttachPoint *>;
 
@@ -609,6 +634,8 @@ std::string opstr(Binop &binop);
 std::string opstr(Unop &unop);
 std::string opstr(Jump &jump);
 
+SizedType ident_to_record(const std::string &ident, int pointer_level = 0);
+
 #undef DEFINE_ACCEPT
 
 } // namespace ast
