@@ -1,15 +1,16 @@
 #include "ast/attachpoint_parser.h"
 
-#include <algorithm>
-#include <exception>
-#include <functional>
-#include <string>
-#include <unordered_map>
-#include <vector>
-#include <bcc/bcc_proc.h>
 #include "ast/int_parser.h"
 #include "log.h"
 #include "types.h"
+#include <algorithm>
+#include <bcc/bcc_proc.h>
+#include <exception>
+#include <functional>
+#include <iostream>
+#include <string>
+#include <unordered_map>
+#include <vector>
 
 namespace bpftrace {
 namespace ast {
@@ -172,13 +173,10 @@ AttachPointParser::State AttachPointParser::parse_attachpoint(AttachPoint &ap)
     return NEW_APS;
   }
 
-  ap_->provider = probetypeName(*probe_types.begin());
+  ap.provider = expand_probe_name(*probe_types.begin());
 
   switch (probetype(ap.provider))
   {
-    case ProbeType::invalid:
-      LOG(FATAL) << "Invalid probe type made it to attachpoint parser";
-      return INVALID;
     case ProbeType::special:
       return special_parser();
     case ProbeType::kprobe:
@@ -210,15 +208,20 @@ AttachPointParser::State AttachPointParser::parse_attachpoint(AttachPoint &ap)
       return kfunc_parser();
     case ProbeType::iter:
       return iter_parser();
+    case ProbeType::rawtracepoint:
+      return raw_tracepoint_parser();
+    case ProbeType::invalid:
+      errs_ << "Invalid probe type: " << ap.provider << std::endl;
+      return INVALID;
   }
 
-  return OK;
+  __builtin_unreachable();
 }
 
 AttachPointParser::State AttachPointParser::lex_attachpoint(
     const AttachPoint &ap)
 {
-  const auto &raw = ap.raw_input;
+  std::string raw = ap.raw_input;
   std::vector<std::string> ret;
   bool in_quotes = false;
   std::string argument;
@@ -266,10 +269,11 @@ AttachPointParser::State AttachPointParser::lex_attachpoint(
         return State::INVALID;
       }
 
-      argument += bpftrace_.get_param(param_idx, true);
-      // NB the for loop will then do idx++ so while we consumed
-      // (len + 1) characters, we only increment by (len).
-      idx += len;
+      // Expand the positional param in-place and decrement idx so that the next
+      // iteration takes the first char of the expansion
+      raw = raw.substr(0, idx) + bpftrace_.get_param(param_idx, true) +
+            raw.substr(i);
+      idx--;
     }
     else
       argument += raw[idx];
@@ -777,5 +781,30 @@ AttachPointParser::State AttachPointParser::iter_parser()
   return OK;
 }
 
+AttachPointParser::State AttachPointParser::raw_tracepoint_parser()
+{
+  if (parts_.size() != 2)
+  {
+    if (ap_->ignore_invalid)
+      return SKIP;
+
+    errs_ << ap_->provider << " probe type requires 1 argument" << std::endl;
+    return INVALID;
+  }
+
+  ap_->func = parts_[1];
+
+  if (has_wildcard(ap_->func))
+  {
+    if (ap_->ignore_invalid)
+      return SKIP;
+    errs_ << ap_->provider << " probe type does not support wildcards"
+          << std::endl;
+    return INVALID;
+  }
+
+  return OK;
+}
+
 } // namespace ast
 } // namespace bpftrace
