@@ -1436,7 +1436,7 @@ CallInst *IRBuilderBPF::CreateSkbOutput(Value *skb,
 
   // long bpf_skb_output(void *skb, struct bpf_map *map, u64 flags,
   //                     void *data, u64 size)
-  FunctionType *skb_output_func_type = FunctionType::get(getInt64Ty(),
+  FunctionType *skb_output_func_type = FunctionType::get(getInt32Ty(),
                                                          { skb->getType(),
                                                            map_ptr->getType(),
                                                            getInt64Ty(),
@@ -1474,6 +1474,21 @@ Value *IRBuilderBPF::CreatKFuncArg(Value *ctx,
   return expr;
 }
 
+Value *IRBuilderBPF::CreateRawTracepointArg(Value *ctx,
+                                            const std::string &builtin)
+{
+  // argX
+  int offset = atoi(builtin.substr(3).c_str());
+  llvm::Type *type = getInt64Ty();
+
+  ctx = CreatePointerCast(ctx, type->getPointerTo());
+  Value *expr = CreateLoad(type,
+                           CreateGEP(type, ctx, getInt64(offset)),
+                           builtin);
+
+  return expr;
+}
+
 Value *IRBuilderBPF::CreateRegisterRead(Value *ctx, const std::string &builtin)
 {
   int offset;
