@@ -37,6 +37,8 @@ public:
   void visit(Builtin &builtin) override;
   void visit(StackMode &) override { };
   void visit(Call &call) override;
+  void visit(Sizeof &szof) override;
+  void visit(Offsetof &ofof) override;
   void visit(Map &map) override;
   void visit(Variable &var) override;
   void visit(Binop &binop) override;
@@ -58,7 +60,7 @@ public:
   void visit(Probe &probe) override;
   void visit(Program &program) override;
   AllocaInst *getHistMapKey(Map &map, Value *log2);
-  int         getNextIndexForProbe(const std::string &probe_name);
+  int getNextIndexForProbe();
   Value      *createLogicalAnd(Binop &binop);
   Value      *createLogicalOr(Binop &binop);
 
@@ -221,7 +223,9 @@ private:
   std::string probefull_;
   std::string tracepoint_struct_;
   uint64_t probe_count_ = 0;
-  std::map<std::string, int> next_probe_index_;
+  // Probes and attach points are indexed from 1, 0 means no index
+  // (no index is used for probes whose attach points are indexed individually)
+  int next_probe_index_ = 1;
   // Used if there are duplicate USDT entries
   int current_usdt_location_index_{ 0 };
 
