@@ -40,6 +40,8 @@ void FieldAnalyser::visit(Builtin &builtin)
         builtin_type = "struct bpf_iter__task";
       else if (attach_func_ == "task_file")
         builtin_type = "struct bpf_iter__task_file";
+      else if (attach_func_ == "task_vma")
+        builtin_type = "struct bpf_iter__task_vma";
     }
   }
   else if (builtin.ident == "curtask")
@@ -103,12 +105,6 @@ void FieldAnalyser::visit(FieldAccess &acc)
   }
   else if (!sized_type_.IsNoneTy())
   {
-    if (sized_type_.IsPtrTy())
-    {
-      sized_type_ = *sized_type_.GetPointeeTy();
-      resolve_fields(sized_type_);
-    }
-
     // If the struct type or the field type has not been resolved, add the type
     // to the BTF set to let ClangParser resolve it
     if (bpftrace_.has_btf_data() && sized_type_.IsRecordTy())
@@ -133,18 +129,21 @@ void FieldAnalyser::visit(FieldAccess &acc)
 void FieldAnalyser::visit(Cast &cast)
 {
   Visit(*cast.expr);
-  sized_type_ = CreateNone();
+  resolve_type(cast.type);
+}
 
-  for (auto &ap : *probe_->attach_points)
-    if (Dwarf *dwarf = bpftrace_.get_dwarf(*ap))
-      sized_type_ = dwarf->get_stype(cast.cast_type);
+void FieldAnalyser::visit(Sizeof &szof)
+{
+  if (szof.expr)
+    Visit(*szof.expr);
+  resolve_type(szof.argtype);
+}
 
-  if (sized_type_.IsNoneTy() && bpftrace_.has_btf_data())
-    sized_type_ = bpftrace_.btf_->get_stype(cast.cast_type);
-
-  // Could not resolve destination type - let ClangParser do it
-  if (sized_type_.IsNoneTy())
-    bpftrace_.btf_set_.insert(cast.cast_type);
+void FieldAnalyser::visit(Offsetof &ofof)
+{
+  if (ofof.expr)
+    Visit(*ofof.expr);
+  resolve_type(ofof.record);
 }
 
 void FieldAnalyser::visit(AssignMapStatement &assignment)
@@ -160,6 +159,16 @@ void FieldAnalyser::visit(AssignVarStatement &assignment)
   var_types_.emplace(assignment.var->ident, sized_type_);
 }
 
+void FieldAnalyser::visit(Unop &unop)
+{
+  Visit(*unop.expr);
+  if (unop.op == Operator::MUL && sized_type_.IsPtrTy())
+  {
+    sized_type_ = *sized_type_.GetPointeeTy();
+    resolve_fields(sized_type_);
+  }
+}
+
 bool FieldAnalyser::compare_args(const ProbeArgs &args1, const ProbeArgs &args2)
 {
   using ProbeArgsValue = ProbeArgs::value_type;
@@ -313,6 +322,29 @@ void FieldAnalyser::resolve_fields(SizedType &type)
     bpftrace_.btf_->resolve_fields(type);
 }
 
+void FieldAnalyser::resolve_type(SizedType &type)
+{
+  sized_type_ = CreateNone();
+
+  const SizedType *inner_type = &type;
+  while (inner_type->IsPtrTy())
+    inner_type = inner_type->GetPointeeTy();
+  if (!inner_type->IsRecordTy())
+    return;
+  auto name = inner_type->GetName();
+
+  for (auto &ap : *probe_->attach_points)
+    if (Dwarf *dwarf = bpftrace_.get_dwarf(*ap))
+      sized_type_ = dwarf->get_stype(name);
+
+  if (sized_type_.IsNoneTy() && bpftrace_.has_btf_data())
+    sized_type_ = bpftrace_.btf_->get_stype(name);
+
+  // Could not resolve destination type - let ClangParser do it
+  if (sized_type_.IsNoneTy())
+    bpftrace_.btf_set_.insert(name);
+}
+
 void FieldAnalyser::visit(Probe &probe)
 {
   probe_ = &probe;
