@@ -32,8 +32,11 @@ public:
   void visit(Variable &var) override;
   void visit(FieldAccess &acc) override;
   void visit(Cast &cast) override;
+  void visit(Sizeof &szof) override;
+  void visit(Offsetof &ofof) override;
   void visit(AssignMapStatement &assignment) override;
   void visit(AssignVarStatement &assignment) override;
+  void visit(Unop &unop) override;
   void visit(Probe &probe) override;
 
   int analyse();
@@ -42,6 +45,7 @@ private:
   bool resolve_args(Probe &probe);
   bool compare_args(const ProbeArgs &args1, const ProbeArgs &args2);
   void resolve_fields(SizedType &type);
+  void resolve_type(SizedType &type);
 
   Node *root_;
   ProbeType probe_type_;
