@@ -80,6 +80,7 @@ libbpf::bpf_prog_type progtype(ProbeType t)
     case ProbeType::kfunc:      return libbpf::BPF_PROG_TYPE_TRACING; break;
     case ProbeType::kretfunc:   return libbpf::BPF_PROG_TYPE_TRACING; break;
     case ProbeType::iter:       return libbpf::BPF_PROG_TYPE_TRACING; break;
+    case ProbeType::rawtracepoint: return libbpf::BPF_PROG_TYPE_RAW_TRACEPOINT; break;
     // clang-format on
     case ProbeType::invalid:
       LOG(FATAL) << "program type invalid";
@@ -147,6 +148,27 @@ int AttachedProbe::detach_iter(void)
   return 0;
 }
 
+void AttachedProbe::attach_raw_tracepoint(void)
+{
+  if (progfd_ < 0)
+    // Errors for raw_tracepoint are handled in load_prog, ignore here
+    return;
+  tracing_fd_ = bpf_raw_tracepoint_open(probe_.attach_point.c_str(), progfd_);
+  if (tracing_fd_ < 0)
+  {
+    if (tracing_fd_ == -ENOENT)
+      throw std::runtime_error("Probe does not exist: " + probe_.name);
+    else
+      throw std::runtime_error("Error attaching probe: " + probe_.name);
+  }
+}
+
+int AttachedProbe::detach_raw_tracepoint(void)
+{
+  close(tracing_fd_);
+  return 0;
+}
+
 AttachedProbe::AttachedProbe(Probe &probe,
                              std::tuple<uint8_t *, uintptr_t> func,
                              bool safe_mode,
@@ -198,6 +220,9 @@ AttachedProbe::AttachedProbe(Probe &probe,
     case ProbeType::iter:
       attach_iter();
       break;
+    case ProbeType::rawtracepoint:
+      attach_raw_tracepoint();
+      break;
     default:
       LOG(FATAL) << "invalid attached probe type \""
                  << probetypeName(probe_.type) << "\"";
@@ -272,6 +297,9 @@ AttachedProbe::~AttachedProbe()
     case ProbeType::asyncwatchpoint:
     case ProbeType::hardware:
       break;
+    case ProbeType::rawtracepoint:
+      err = detach_raw_tracepoint();
+      break;
     case ProbeType::invalid:
       LOG(FATAL) << "invalid attached probe type \""
                  << probetypeName(probe_.type) << "\" at destructor";
@@ -314,6 +342,7 @@ std::string AttachedProbe::eventname() const
   {
     case ProbeType::kprobe:
     case ProbeType::kretprobe:
+    case ProbeType::rawtracepoint:
       offset_str << std::hex << offset_;
       return eventprefix() + sanitise(probe_.attach_point) + "_" +
              offset_str.str() + index_str;
@@ -440,7 +469,7 @@ static void check_alignment(std::string &path,
   }
 }
 
-void AttachedProbe::resolve_offset_uprobe(bool safe_mode)
+bool AttachedProbe::resolve_offset_uprobe(bool safe_mode)
 {
   struct bcc_symbol_option option = { };
   struct symbol sym = { };
@@ -472,7 +501,7 @@ void AttachedProbe::resolve_offset_uprobe(bool safe_mode)
                      << " (binary appears stripped). Misaligned probes "
                         "can lead to tracee crashes!";
         offset_ = probe_.address;
-        return;
+        return true;
       }
     }
 
@@ -495,10 +524,33 @@ void AttachedProbe::resolve_offset_uprobe(bool safe_mode)
     throw std::runtime_error(msg.str());
   }
 
-  if (func_offset >= sym.size) {
+  if (sym.size == 0 && func_offset == 0)
+  {
+    if (safe_mode)
+    {
+      std::stringstream msg;
+      msg << "Could not determine boundary for " << sym.name
+          << " (symbol has size 0).";
+      if (probe_.orig_name == probe_.name)
+      {
+        msg << " Use --unsafe to force attachment.";
+        throw std::runtime_error(msg.str());
+      }
+      else
+      {
+        LOG(WARNING)
+            << msg.str()
+            << " Skipping attachment (use --unsafe to force attachement).";
+      }
+      return false;
+    }
+  }
+  else if (func_offset >= sym.size)
+  {
     std::stringstream ss;
     ss << sym.size;
-    throw std::runtime_error("Offset outside the function bounds ('" + symbol + "' size is " + ss.str() + ")");
+    throw std::runtime_error("Offset outside the function bounds ('" + symbol +
+                             "' size is " + ss.str() + ")");
   }
 
   uint64_t sym_offset = resolve_offset(probe_.path, probe_.attach_point, probe_.loc);
@@ -507,10 +559,11 @@ void AttachedProbe::resolve_offset_uprobe(bool safe_mode)
   // If we are not aligned to the start of the symbol,
   // check if we are on the instruction boundary.
   if (func_offset == 0)
-    return;
+    return true;
 
   check_alignment(
       probe_.path, symbol, sym_offset, func_offset, safe_mode, probe_.type);
+  return true;
 }
 
 // find vmlinux file containing the given symbol information
@@ -762,12 +815,13 @@ void AttachedProbe::load_prog(BPFfeature &feature)
   }
 
   if (progfd_ < 0) {
+    std::stringstream errmsg;
     if (bt_verbose) {
       std::cerr << std::endl
                 << "Error log: " << std::endl
                 << log_buf.get() << std::endl;
-      if (errno == ENOSPC) {
-        std::stringstream errmsg;
+      if (errno == ENOSPC)
+      {
         errmsg << "Error: Failed to load program, verification log buffer "
                << "not big enough, try increasing the BPFTRACE_LOG_SIZE "
                << "environment variable beyond the current value of "
@@ -791,7 +845,18 @@ void AttachedProbe::load_prog(BPFfeature &feature)
       }
     }
 
-    throw std::runtime_error("Error loading program: " + probe_.name + (bt_verbose ? "" : " (try -v)"));
+    errmsg << "Error loading program: " << probe_.name
+           << (bt_verbose ? "" : " (try -v)");
+    if (probe_.orig_name != probe_.name)
+    {
+      // one attachpoint in a multi-attachpoint (wildcard or list) probe failed,
+      // print a warning but continue
+      LOG(WARNING) << errmsg.str() << ", skipping.";
+      return;
+    }
+    else
+      // explicit match failed, fail hard
+      throw std::runtime_error(errmsg.str());
   }
 
   if (bt_verbose) {
@@ -893,7 +958,8 @@ void AttachedProbe::attach_kprobe(bool safe_mode)
 
 void AttachedProbe::attach_uprobe(bool safe_mode)
 {
-  resolve_offset_uprobe(safe_mode);
+  if (!resolve_offset_uprobe(safe_mode))
+    return;
 
   int perf_event_fd = bpf_attach_uprobe(progfd_,
                                         attachtype(probe_.type),
@@ -1052,11 +1118,12 @@ void AttachedProbe::attach_usdt(int pid, BPFfeature &feature)
 
   if (err)
   {
-    std::string err;
-    err += "Error finding or enabling probe: " + probe_.name;
-    err += '\n';
-    err += "Try using -p or --usdt-file-activation if there's USDT semaphores";
-    throw std::runtime_error(err);
+    std::string errstr;
+    errstr += "Error finding or enabling probe: " + probe_.name;
+    errstr += '\n';
+    errstr +=
+        "Try using -p or --usdt-file-activation if there's USDT semaphores";
+    throw std::runtime_error(errstr);
   }
 
   int perf_event_fd =
