@@ -10,6 +10,7 @@
 #include <unistd.h>
 
 #include "btf.h"
+#include "debugfs.h"
 #include "probe_matcher.h"
 #include "tracefs.h"
 #include "utils.h"
@@ -86,7 +87,12 @@ bool BPFfeature::try_load(enum libbpf::bpf_prog_type prog_type,
 
   std::optional<unsigned> btf_id;
   if (prog_type == libbpf::BPF_PROG_TYPE_TRACING && has_btf())
-    btf_id = btf_.get_btf_id_fd(name, "vmlinux").first;
+  {
+    auto id_fd = btf_.get_btf_id_fd(name, "vmlinux");
+    btf_id = id_fd.first;
+    if (id_fd.second >= 0)
+      close(id_fd.second);
+  }
 
   if (prog_type == libbpf::BPF_PROG_TYPE_TRACING)
   {
@@ -150,10 +156,12 @@ bool BPFfeature::detect_helper(enum libbpf::bpf_func_id func_id,
 bool BPFfeature::detect_prog_type(
     enum libbpf::bpf_prog_type prog_type,
     const char* name,
-    std::optional<libbpf::bpf_attach_type> attach_type)
+    std::optional<libbpf::bpf_attach_type> attach_type,
+    int* outfd)
 {
   struct bpf_insn insns[] = { BPF_MOV64_IMM(BPF_REG_0, 0), BPF_EXIT_INSN() };
-  return try_load(prog_type, insns, ARRAY_SIZE(insns), name, attach_type);
+  return try_load(
+      prog_type, insns, ARRAY_SIZE(insns), name, attach_type, outfd);
 }
 
 bool BPFfeature::detect_map(enum libbpf::bpf_map_type map_type)
@@ -426,7 +434,8 @@ bool BPFfeature::has_skb_output(void)
       try_load(libbpf::BPF_PROG_TYPE_TRACING,
                insns,
                ARRAY_SIZE(insns),
-               "kfunc__kfree_skb"));
+               "__kfree_skb",
+               libbpf::BPF_TRACE_FENTRY));
 
   close(map_fd);
   return *has_skb_output_;
@@ -490,6 +499,7 @@ std::string BPFfeature::report(void)
       << "  Instruction limit: " << instruction_limit() << std::endl
       << "  Loop support: " << to_str(has_loop())
       << "  btf: " << to_str(has_btf())
+      << "  module btf: " << to_str(has_module_btf())
       << "  map batch: " << to_str(has_map_batch())
       << "  uprobe refcount (depends on Build:bcc bpf_attach_uprobe refcount): "
       << to_str(has_uprobe_refcnt()) << std::endl;
@@ -510,15 +520,72 @@ std::string BPFfeature::report(void)
       << "  kfunc: " << to_str(has_kfunc())
       << "  iter:task: " << to_str(has_prog_iter_task())
       << "  iter:task_file: " << to_str(has_prog_iter_task_file())
+      << "  iter:task_vma: " << to_str(has_prog_iter_task_vma())
       << "  kprobe_multi: " << to_str(has_kprobe_multi())
       << "  raw_tp_special: " << to_str(has_raw_tp_special()) << std::endl;
 
   return buf.str();
 }
 
+bool BPFfeature::has_prog_kfunc()
+{
+  if (!has_prog_kfunc_.has_value())
+  {
+    int progfd;
+    if (!detect_prog_type(libbpf::BPF_PROG_TYPE_TRACING,
+                          "sched_fork",
+                          libbpf::BPF_TRACE_FENTRY,
+                          &progfd))
+      goto out_false;
+    int tracing_fd = bpf_raw_tracepoint_open(nullptr, progfd);
+    close(progfd);
+    if (tracing_fd < 0)
+      goto out_false;
+    close(tracing_fd);
+    has_prog_kfunc_ = std::make_optional<bool>(true);
+  }
+  return *(has_prog_kfunc_);
+out_false:
+  has_prog_kfunc_ = std::make_optional<bool>(false);
+  return *(has_prog_kfunc_);
+}
+
 bool BPFfeature::has_kfunc()
 {
   return has_prog_kfunc() && btf_.has_data();
 }
 
+bool BPFfeature::has_module_btf()
+{
+  if (has_module_btf_.has_value())
+    return *has_module_btf_;
+
+  char name[64];
+  struct bpf_btf_info info = {};
+  info.name = (__u64)name;
+  info.name_len = sizeof(name);
+  __u32 id = 0, info_len = sizeof(info);
+  int err = 0, fd = -1;
+
+  err = bpf_btf_get_next_id(id, &id);
+  if (err)
+    goto not_support;
+
+  fd = bpf_btf_get_fd_by_id(id);
+  if (fd < 0)
+    goto not_support;
+
+  err = bpf_obj_get_info_by_fd(fd, &info, &info_len);
+  close(fd);
+  if (err)
+    goto not_support;
+
+  has_module_btf_ = true;
+  return *has_module_btf_;
+
+not_support:
+  has_module_btf_ = false;
+  return *has_module_btf_;
+}
+
 } // namespace bpftrace
