@@ -83,6 +83,8 @@ public:
   bool has_kfunc();
   bool has_skb_output();
   bool has_raw_tp_special();
+  bool has_prog_kfunc();
+  bool has_module_btf();
 
   std::string report(void);
 
@@ -105,10 +107,6 @@ public:
   DEFINE_PROG_TEST(kprobe, libbpf::BPF_PROG_TYPE_KPROBE);
   DEFINE_PROG_TEST(tracepoint, libbpf::BPF_PROG_TYPE_TRACEPOINT);
   DEFINE_PROG_TEST(perf_event, libbpf::BPF_PROG_TYPE_PERF_EVENT);
-  DEFINE_PROG_TEST_FUNC(kfunc,
-                        libbpf::BPF_PROG_TYPE_TRACING,
-                        "sched_fork",
-                        libbpf::BPF_TRACE_FENTRY);
   DEFINE_PROG_TEST_FUNC(iter_task,
                         libbpf::BPF_PROG_TYPE_TRACING,
                         "bpf_iter_task",
@@ -117,6 +115,10 @@ public:
                         libbpf::BPF_PROG_TYPE_TRACING,
                         "bpf_iter_task_file",
                         libbpf::BPF_TRACE_ITER);
+  DEFINE_PROG_TEST_FUNC(iter_task_vma,
+                        libbpf::BPF_PROG_TYPE_TRACING,
+                        "bpf_iter_task_vma",
+                        libbpf::BPF_TRACE_ITER);
 
 protected:
   std::optional<bool> has_loop_;
@@ -127,6 +129,8 @@ protected:
   std::optional<bool> has_kprobe_multi_;
   std::optional<bool> has_skb_output_;
   std::optional<bool> has_raw_tp_special_;
+  std::optional<bool> has_prog_kfunc_;
+  std::optional<bool> has_module_btf_;
 
 private:
   bool detect_map(libbpf::bpf_map_type map_type);
@@ -134,7 +138,8 @@ private:
                      libbpf::bpf_prog_type prog_type);
   bool detect_prog_type(libbpf::bpf_prog_type prog_type,
                         const char* name,
-                        std::optional<libbpf::bpf_attach_type> attach_type);
+                        std::optional<libbpf::bpf_attach_type> attach_type,
+                        int* outfd = nullptr);
 
   bool try_load(
       libbpf::bpf_prog_type prog_type,
