@@ -1,5 +1,6 @@
 #pragma once
 
+#include <cstdint>
 #include <iostream>
 #include <map>
 #include <memory>
@@ -139,7 +140,7 @@ public:
   void request_finalize();
   bool is_aslr_enabled(int pid);
   std::string get_string_literal(const ast::Expression *expr) const;
-  std::optional<long> get_int_literal(const ast::Expression *expr) const;
+  std::optional<int64_t> get_int_literal(const ast::Expression *expr) const;
   std::optional<std::string> get_watchpoint_binary_path() const;
   virtual bool is_traceable_func(const std::string &func_name) const;
   std::unordered_set<std::string> get_func_modules(
@@ -168,6 +169,7 @@ public:
   std::map<libbpf::bpf_func_id, location> helper_use_loc_;
   // mapping traceable functions to modules (or "vmlinux") that they appear in
   FuncsModulesMap traceable_funcs_;
+  KConfig kconfig;
   std::vector<std::unique_ptr<AttachedProbe>> attached_probes_;
 
   std::map<std::string, std::unique_ptr<PCAPwriter>> pcap_writers;
@@ -178,6 +180,7 @@ public:
   std::unique_ptr<BPFfeature> feature_;
 
   uint64_t strlen_ = 64;
+  const char *str_trunc_trailer_ = "..";
   uint64_t mapmax_ = 4096;
   size_t cat_bytes_max_ = 10240;
   uint64_t max_probes_ = 512;
@@ -193,6 +196,7 @@ public:
   bool usdt_file_activation_ = false;
   int helper_check_level_ = 0;
   uint64_t ast_max_nodes_ = 0; // Maximum AST nodes allowed for fuzzing
+  std::optional<StackMode> stack_mode_;
   std::optional<struct timespec> boottime_;
 
   static void sort_by_key(
