@@ -72,6 +72,19 @@ BTF::~BTF()
 
 void BTF::load_kernel_btfs(const std::set<std::string> &modules)
 {
+  vmlinux_btf = btf__load_vmlinux_btf();
+  if (libbpf_get_error(vmlinux_btf))
+  {
+    if (bt_debug != DebugLevel::kNone)
+      LOG(ERROR) << "BTF: failed to find BTF data for vmlinux, errno " << errno;
+    return;
+  }
+  btf_objects.push_back(
+      BTFObj{ .btf = vmlinux_btf, .id = 0, .name = "vmlinux" });
+
+  if (bpftrace_ && !bpftrace_->feature_->has_module_btf())
+    return;
+
   // Note that we cannot parse BTFs from /sys/kernel/btf/ as we need BTF object
   // IDs, so the only way is to iterate through all loaded BTF objects
   __u32 id = 0;
@@ -116,19 +129,10 @@ void BTF::load_kernel_btfs(const std::set<std::string> &modules)
 
     if (mod_name == "vmlinux")
     {
-      btf_objects.push_back(BTFObj{ .btf = btf__load_from_kernel_by_id(id),
-                                    .id = id,
-                                    .name = "vmlinux" });
-      vmlinux_btf = btf_objects.back().btf;
+      btf_objects.front().id = id;
     }
-    else if (modules.empty() || modules.find(mod_name) != modules.end())
+    else if (modules.find(mod_name) != modules.end())
     {
-      if (!vmlinux_btf)
-      {
-        if (bt_debug != DebugLevel::kNone)
-          LOG(ERROR) << "BTF: failed to find BTF data for vmlinux";
-        return;
-      }
       btf_objects.push_back(
           BTFObj{ .btf = btf__load_from_kernel_by_id_split(id, vmlinux_btf),
                   .id = id,
@@ -250,9 +254,16 @@ std::string BTF::c_def(const std::unordered_set<std::string> &set) const
   if (!has_data())
     return std::string("");
 
-  // Definition dumping from modules would require to resolve type conflicts,
-  // so we dump from vmlinux only for now.
+  // Definition dumping from multiple modules would require to resolve type
+  // conflicts, so we allow dumping from a single module or from vmlinux only.
   std::unordered_set<std::string> to_dump(set);
+  if (btf_objects.size() == 2)
+  {
+    auto *mod_btf = btf_objects[0].btf == vmlinux_btf ? btf_objects[1].btf
+                                                      : btf_objects[0].btf;
+    return dump_defs_from_btf(mod_btf, to_dump);
+  }
+
   return dump_defs_from_btf(vmlinux_btf, to_dump);
 }
 
@@ -715,7 +726,7 @@ std::pair<int, int> BTF::get_btf_id_fd(const std::string &func,
 
     auto id = find_id_in_btf(btf_obj.btf, func, BTF_KIND_FUNC);
     if (id >= 0)
-      return { id, bpf_btf_get_fd_by_id(btf_obj.id) };
+      return { id, btf_obj.id ? bpf_btf_get_fd_by_id(btf_obj.id) : 0 };
   }
 
   return { -1, -1 };
@@ -742,6 +753,8 @@ __s32 BTF::find_id_in_btf(struct btf *btf,
   for (__s32 id = start_id(btf), max = (__s32)type_cnt(btf); id <= max; ++id)
   {
     const struct btf_type *t = btf__type_by_id(btf, id);
+    if (!t)
+      continue;
     if (kind && btf_kind(t) != *kind)
       continue;
 
@@ -771,11 +784,25 @@ void BTF::resolve_fields(SizedType &type)
   resolve_fields(type_id, record.get(), 0);
 }
 
+static std::optional<Bitfield> resolve_bitfield(
+    const struct btf_type *record_type,
+    __u32 member_idx)
+{
+  __u32 bitfield_width = btf_member_bitfield_size(record_type, member_idx);
+  if (bitfield_width <= 0)
+    return std::nullopt;
+
+  return Bitfield(btf_member_bit_offset(record_type, member_idx) % 8,
+                  bitfield_width);
+}
+
 void BTF::resolve_fields(const BTFId &type_id,
                          Struct *record,
                          __u32 start_offset)
 {
   auto btf_type = btf__type_by_id(type_id.btf, type_id.id);
+  if (!btf_type)
+    return;
   auto members = btf_members(btf_type);
   for (__u32 i = 0; i < BTF_INFO_VLEN(btf_type->info); i++)
   {
@@ -791,13 +818,8 @@ void BTF::resolve_fields(const BTFId &type_id,
 
     std::string field_name = btf__name_by_offset(type_id.btf,
                                                  members[i].name_off);
-    if (members[i].offset % 8 != 0)
-    {
-      // bitfield - not supported yet
-      record->ClearFields();
-      return;
-    }
-    __u32 field_offset = start_offset + members[i].offset / 8;
+
+    __u32 field_offset = start_offset + btf_member_bit_offset(btf_type, i) / 8;
 
     if (btf_is_composite(field_type) &&
         (field_name.empty() || field_name == "(anon)"))
@@ -806,8 +828,11 @@ void BTF::resolve_fields(const BTFId &type_id,
       return;
     }
 
-    record->AddField(
-        field_name, get_stype(field_id), field_offset, false, {}, false);
+    record->AddField(field_name,
+                     get_stype(field_id),
+                     field_offset,
+                     resolve_bitfield(btf_type, i),
+                     false);
   }
 }
 
