@@ -50,7 +50,6 @@ enum class OutputBufferConfig {
 enum class TestMode
 {
   UNSET = 0,
-  SEMANTIC,
   CODEGEN,
 };
 
@@ -128,6 +127,8 @@ void usage()
   std::cerr << "    BPFTRACE_CACHE_USER_SYMBOLS [default: auto] enable user symbol cache" << std::endl;
   std::cerr << "    BPFTRACE_VMLINUX            [default: none] vmlinux path used for kernel symbol resolution" << std::endl;
   std::cerr << "    BPFTRACE_BTF                [default: none] BTF file" << std::endl;
+  std::cerr << "    BPFTRACE_STR_TRUNC_TRAILER  [default: '..'] string truncation trailer" << std::endl;
+  std::cerr << "    BPFTRACE_STACK_MODE         [default: bpftrace] Output format for ustack and kstack builtins" << std::endl;
   std::cerr << std::endl;
   std::cerr << "EXAMPLES:" << std::endl;
   std::cerr << "bpftrace -l '*sleep*'" << std::endl;
@@ -249,6 +250,9 @@ static std::optional<struct timespec> get_boottime()
   if (!get_uint64_env_var("BPFTRACE_STRLEN", bpftrace.strlen_))
     return false;
 
+  if (const char* env_p = std::getenv("BPFTRACE_STR_TRUNC_TRAILER"))
+    bpftrace.str_trunc_trailer_ = env_p;
+
   // in practice, the largest buffer I've seen fit into the BPF stack was 240
   // bytes. I've set the bar lower, in case your program has a deeper stack than
   // the one from my tests, in the hope that you'll get this instructive error
@@ -331,6 +335,21 @@ static std::optional<struct timespec> get_boottime()
   if (!get_bool_env_var("BPFTRACE_VERIFY_LLVM_IR", verify_llvm_ir))
     return false;
 
+  if (const char* stack_mode = std::getenv("BPFTRACE_STACK_MODE"))
+  {
+    auto found = STACK_MODE_MAP.find(stack_mode);
+    if (found != STACK_MODE_MAP.end())
+    {
+      bpftrace.stack_mode_ = found->second;
+    }
+    else
+    {
+      LOG(ERROR) << "Env var 'BPFTRACE_STACK_MODE' did not contain a valid "
+                    "StackMode: "
+                 << stack_mode;
+    }
+  }
+
   return true;
 }
 
@@ -370,7 +389,8 @@ static std::optional<struct timespec> get_boottime()
     kobj = std::get<1>(kdirs);
 
     if (ksrc != "")
-      extra_flags = get_kernel_cflags(utsname.machine, ksrc, kobj);
+      extra_flags = get_kernel_cflags(
+          utsname.machine, ksrc, kobj, bpftrace.kconfig);
   }
   extra_flags.push_back("-include");
   extra_flags.push_back(CLANG_WORKAROUNDS_H);
@@ -413,7 +433,7 @@ ast::PassManager CreateDynamicPM()
   return pm;
 }
 
-ast::PassManager CreateAotPM(std::string __attribute__((unused)))
+ast::PassManager CreateAotPM()
 {
   ast::PassManager pm;
   pm.AddPass(ast::CreateSemanticPass());
@@ -493,13 +513,11 @@ Args parse_args(int argc, char* argv[])
         DISABLE_LOG(WARNING);
         break;
       case Options::TEST: // --test
-        if (std::strcmp(optarg, "semantic") == 0)
-          args.test_mode = TestMode::SEMANTIC;
-        else if (std::strcmp(optarg, "codegen") == 0)
+        if (std::strcmp(optarg, "codegen") == 0)
           args.test_mode = TestMode::CODEGEN;
         else
         {
-          LOG(ERROR) << "USAGE: --test must be either 'semantic' or 'codegen'.";
+          LOG(ERROR) << "USAGE: --test can only be 'codegen'.";
           exit(1);
         }
         break;
@@ -666,11 +684,27 @@ Args parse_args(int argc, char* argv[])
   return args;
 }
 
+static const char* libbpf_print_level_string(enum libbpf_print_level level)
+{
+  switch (level)
+  {
+    case LIBBPF_WARN:
+      return "WARN";
+    case LIBBPF_INFO:
+      return "INFO";
+    default:
+      return "DEBUG";
+  }
+}
+
 static int libbpf_print(enum libbpf_print_level level,
                         const char* msg,
                         va_list ap)
 {
-  fprintf(stderr, "libbpf: (%d) ", level);
+  if (bt_debug == DebugLevel::kNone)
+    return 0;
+
+  fprintf(stderr, "[%s] ", libbpf_print_level_string(level));
   return vfprintf(stderr, msg, ap);
 }
 
@@ -722,8 +756,7 @@ int main(int argc, char* argv[])
       break;
   }
 
-  if (bt_debug != DebugLevel::kNone)
-    libbpf_set_print(libbpf_print);
+  libbpf_set_print(libbpf_print);
 
   BPFtrace bpftrace(std::move(output));
   bool verify_llvm_ir = false;
@@ -853,7 +886,7 @@ int main(int argc, char* argv[])
   if (!is_root())
     return 1;
 
-  auto lockdown_state = lockdown::detect(bpftrace.feature_);
+  auto lockdown_state = lockdown::detect();
   if (lockdown_state == lockdown::LockdownState::Confidentiality)
   {
     lockdown::emit_warning(std::cerr);
@@ -877,7 +910,7 @@ int main(int argc, char* argv[])
       pm = CreateDynamicPM();
       break;
     case BuildMode::AHEAD_OF_TIME:
-      pm = CreateAotPM(args.aot);
+      pm = CreateAotPM();
       break;
   }
 
