@@ -282,6 +282,12 @@ std::string Output::value_to_str(BPFtrace &bpftrace,
         reinterpret_cast<AsyncEvent::CgroupPath *>(value.data())->cgroup_id);
   else if (type.IsStrerrorTy())
     return strerror(read_data<uint64_t>(value.data()));
+  else if (type.IsPtrTy())
+  {
+    std::ostringstream res;
+    res << "0x" << std::hex << read_data<uint64_t>(value.data());
+    return res.str();
+  }
   else
     return std::to_string(read_data<int64_t>(value.data()) / div);
 }
