@@ -1,8 +1,51 @@
 #include "struct.h"
+
 #include <iomanip>
+#include <limits>
+
+#include "log.h"
 
 namespace bpftrace {
 
+const size_t BIFTIELD_BIT_WIDTH_MAX = sizeof(uint64_t) * 8;
+
+Bitfield::Bitfield(size_t bit_offset, size_t bit_width)
+{
+  // To handle bitfields, we need to give codegen 3 additional pieces
+  // of information: `read_bytes`, `access_rshift`, and `mask`.
+  //
+  // `read_bytes` tells codegen how many bytes to read starting at
+  // `Field::offset`. This information is necessary because we can't always
+  // issue, for example, a 1 byte read, as the bitfield could be the last 4 bits
+  // of the struct. Reading past the end of the struct could cause a page fault.
+  // Therefore, we compute the minimum number of bytes necessary to fully read
+  // the bitfield. This will always keep the read within the bounds of the
+  // struct.
+  //
+  // `access_rshift` tells codegen how much to shift the masked value so that
+  // the LSB of the bitfield is the LSB of the interpreted integer.
+  //
+  // `mask` tells codegen how to mask out the surrounding bitfields.
+
+  if (bit_width > BIFTIELD_BIT_WIDTH_MAX)
+  {
+    LOG(WARNING) << "bitfield bitwidth " << bit_width << "is not supported."
+                 << " Use bitwidth " << BIFTIELD_BIT_WIDTH_MAX;
+    bit_width = BIFTIELD_BIT_WIDTH_MAX;
+  }
+  if (bit_width == BIFTIELD_BIT_WIDTH_MAX)
+    mask = std::numeric_limits<uint64_t>::max();
+  else
+    mask = (1ULL << bit_width) - 1;
+  // Round up to nearest byte
+  read_bytes = (bit_offset + bit_width + 7) / 8;
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+  access_rshift = bit_offset;
+#else
+  access_rshift = (read_bytes * 8 - bit_offset - bit_width);
+#endif
+}
+
 bool Bitfield::operator==(const Bitfield &other) const {
   return read_bytes == other.read_bytes
     && mask == other.mask
@@ -35,8 +78,7 @@ std::unique_ptr<Struct> Struct::CreateTuple(std::vector<SizedType> fields)
         .name = "",
         .type = field,
         .offset = offset,
-        .is_bitfield = false,
-        .bitfield = Bitfield{},
+        .bitfield = std::nullopt,
     });
 
     offset += size;
@@ -100,25 +142,35 @@ const Field &Struct::GetField(const std::string &name) const
 void Struct::AddField(const std::string &field_name,
                       const SizedType &type,
                       ssize_t offset,
-                      bool is_bitfield,
-                      const Bitfield &bitfield,
+                      const std::optional<Bitfield> &bitfield,
                       bool is_data_loc)
 {
   if (!HasField(field_name))
     fields.emplace_back(Field{ .name = field_name,
                                .type = type,
                                .offset = offset,
-                               .is_bitfield = is_bitfield,
                                .bitfield = bitfield,
                                .is_data_loc = is_data_loc });
 }
 
-void StructManager::Add(const std::string &name, size_t size)
+bool Struct::HasFields() const
+{
+  return !fields.empty();
+}
+
+void Struct::ClearFields()
+{
+  fields.clear();
+}
+
+void StructManager::Add(const std::string &name,
+                        size_t size,
+                        bool allow_override)
 {
   if (struct_map_.find(name) != struct_map_.end())
     throw std::runtime_error("Type redefinition: type with name \'" + name +
                              "\' already exists");
-  struct_map_[name] = std::make_unique<Struct>(size);
+  struct_map_[name] = std::make_unique<Struct>(size, allow_override);
 }
 
 std::weak_ptr<Struct> StructManager::Lookup(const std::string &name) const
@@ -128,9 +180,11 @@ std::weak_ptr<Struct> StructManager::Lookup(const std::string &name) const
 }
 
 std::weak_ptr<Struct> StructManager::LookupOrAdd(const std::string &name,
-                                                 size_t size)
+                                                 size_t size,
+                                                 bool allow_override)
 {
-  auto s = struct_map_.insert({ name, std::make_unique<Struct>(size) });
+  auto s = struct_map_.insert(
+      { name, std::make_unique<Struct>(size, allow_override) });
   return s.first->second;
 }
 
