@@ -16,7 +16,6 @@ namespace bpftrace {
 
 const int MAX_STACK_SIZE = 1024;
 const int DEFAULT_STACK_SIZE = 127;
-const int STRING_SIZE = 64;
 const int COMM_SIZE = 16;
 
 enum class Type
@@ -67,6 +66,13 @@ enum class StackMode
 {
   bpftrace,
   perf,
+  raw,
+};
+
+const std::map<std::string, StackMode> STACK_MODE_MAP = {
+  { "bpftrace", StackMode::bpftrace },
+  { "perf", StackMode::perf },
+  { "raw", StackMode::raw },
 };
 
 struct StackType
@@ -200,8 +206,7 @@ public:
 
   bool IsPrintableTy()
   {
-    return type != Type::none && type != Type::pointer &&
-           type != Type::stack_mode && !IsCtxAccess();
+    return type != Type::none && type != Type::stack_mode && !IsCtxAccess();
   }
 
   bool IsSigned(void) const;
@@ -454,6 +459,7 @@ enum class ProbeType
   kfunc,
   kretfunc,
   iter,
+  rawtracepoint,
 };
 
 std::ostream &operator<<(std::ostream &os, ProbeType type);
@@ -483,13 +489,14 @@ const std::vector<ProbeItem> PROBE_LIST = {
   { "kfunc", "f", ProbeType::kfunc },
   { "kretfunc", "fr", ProbeType::kretfunc },
   { "iter", "it", ProbeType::iter },
+  { "rawtracepoint", "rt", ProbeType::rawtracepoint },
 };
 
 ProbeType probetype(const std::string &type);
 bool is_userspace_probe(const ProbeType &probe_type);
 std::string addrspacestr(AddrSpace as);
 std::string typestr(Type t);
-std::string probetypeName(const std::string &type);
+std::string expand_probe_name(const std::string &orig_name);
 std::string probetypeName(ProbeType t);
 
 struct Probe
@@ -592,6 +599,8 @@ struct hash<bpftrace::StackType>
         return std::hash<std::string>()("bpftrace#" + to_string(obj.limit));
       case bpftrace::StackMode::perf:
         return std::hash<std::string>()("perf#" + to_string(obj.limit));
+      case bpftrace::StackMode::raw:
+        return std::hash<std::string>()("raw#" + to_string(obj.limit));
     }
 
     return {}; // unreached
