@@ -16,6 +16,8 @@ namespace bpftrace {
 using ::testing::ContainerEq;
 using ::testing::StrictMock;
 
+static const int STRING_SIZE = 64;
+
 static const std::string kprobe_name(const std::string &attach_point,
                                      uint64_t func_offset)
 {
@@ -116,6 +118,16 @@ void check_tracepoint(Probe &p, const std::string &target, const std::string &fu
   EXPECT_EQ("tracepoint:" + target + ":" + func, p.name);
 }
 
+void check_rawtracepoint(Probe &p,
+                         const std::string &func,
+                         const std::string &orig_name)
+{
+  EXPECT_EQ(ProbeType::rawtracepoint, p.type);
+  EXPECT_EQ(func, p.attach_point);
+  EXPECT_EQ(orig_name, p.orig_name);
+  EXPECT_EQ("rawtracepoint:" + func, p.name);
+}
+
 void check_profile(Probe &p, const std::string &unit, int freq, const std::string &orig_name)
 {
   EXPECT_EQ(ProbeType::profile, p.type);
@@ -210,8 +222,7 @@ TEST(bpftrace, add_probes_wildcard)
       "kprobe:sys_read,kprobe:my_*,kprobe:sys_write{}");
 
   auto bpftrace = get_strict_mock_bpftrace();
-  EXPECT_CALL(*bpftrace->mock_probe_matcher,
-              get_symbols_from_file(tracefs::available_filter_functions()))
+  EXPECT_CALL(*bpftrace->mock_probe_matcher, get_symbols_from_traceable_funcs())
       .Times(1);
 
   if (bpftrace->has_kprobe_multi())
@@ -247,8 +258,7 @@ TEST(bpftrace, add_probes_wildcard_no_matches)
       "kprobe:sys_read,kprobe:not_here_*,kprobe:sys_write{}");
 
   auto bpftrace = get_strict_mock_bpftrace();
-  EXPECT_CALL(*bpftrace->mock_probe_matcher,
-              get_symbols_from_file(tracefs::available_filter_functions()))
+  EXPECT_CALL(*bpftrace->mock_probe_matcher, get_symbols_from_traceable_funcs())
       .Times(1);
 
   ASSERT_EQ(0, bpftrace->add_probe(*probe));
@@ -277,8 +287,7 @@ TEST(bpftrace, add_probes_kernel_module_wildcard)
 {
   ast::Probe *probe = parse_probe("kprobe:func_in_mo*{}");
   auto bpftrace = get_strict_mock_bpftrace();
-  EXPECT_CALL(*bpftrace->mock_probe_matcher,
-              get_symbols_from_file(tracefs::available_filter_functions()))
+  EXPECT_CALL(*bpftrace->mock_probe_matcher, get_symbols_from_traceable_funcs())
       .Times(1);
 
   ASSERT_EQ(0, bpftrace->add_probe(*probe));
@@ -1016,6 +1025,34 @@ TEST_F(bpftrace_btf, add_probes_iter_task_file)
   check_probe(bpftrace.get_probes().at(0), ProbeType::iter, "iter:task_file");
 }
 
+TEST_F(bpftrace_btf, add_probes_iter_task_vma)
+{
+  ast::Probe *probe = parse_probe("iter:task_vma {}");
+
+  StrictMock<MockBPFtrace> bpftrace;
+
+  ASSERT_EQ(0, bpftrace.add_probe(*probe));
+  ASSERT_EQ(1U, bpftrace.get_probes().size());
+  ASSERT_EQ(0U, bpftrace.get_special_probes().size());
+
+  check_probe(bpftrace.get_probes().at(0), ProbeType::iter, "iter:task_vma");
+}
+
+TEST(bpftrace, add_probes_rawtracepoint)
+{
+  auto probe = parse_probe(("rawtracepoint:sched_switch {}"));
+  StrictMock<MockBPFtrace> bpftrace;
+
+  ASSERT_EQ(0, bpftrace.add_probe(*probe));
+  ASSERT_EQ(1U, bpftrace.get_probes().size());
+  ASSERT_EQ(0U, bpftrace.get_special_probes().size());
+
+  std::string probe_orig_name = "rawtracepoint:sched_switch";
+  check_rawtracepoint(bpftrace.get_probes().at(0),
+                      "sched_switch",
+                      probe_orig_name);
+}
+
 } // namespace bpftrace
 } // namespace test
 } // namespace bpftrace
