@@ -14,6 +14,7 @@ TEST(codegen, call_ustack)
   // program
   test("kprobe:f { @x = ustack(); @y = ustack(6) }", result);
   test("kprobe:f { @x = ustack(perf); @y = ustack(perf, 6) }", result);
+  test("kprobe:f { @x = ustack(raw); @y = ustack(raw, 6) }", result);
   test("kprobe:f { @x = ustack(perf); @y = ustack(bpftrace) }", result);
 }
 
@@ -27,19 +28,21 @@ TEST(codegen, call_ustack_mapids)
             0);
 
   ClangParser clang;
-  clang.parse(driver.root_, *bpftrace);
+  clang.parse(driver.root.get(), *bpftrace);
 
   // Override to mockbpffeature.
   bpftrace->feature_ = std::make_unique<MockBPFfeature>(true);
-  ast::SemanticAnalyser semantics(driver.root_, *bpftrace);
+  ast::SemanticAnalyser semantics(driver.root.get(), *bpftrace);
   ASSERT_EQ(semantics.analyse(), 0);
 
-  ast::ResourceAnalyser resource_analyser(driver.root_);
-  auto resources = resource_analyser.analyse();
+  ast::ResourceAnalyser resource_analyser(driver.root.get());
+  auto resources_optional = resource_analyser.analyse();
+  ASSERT_TRUE(resources_optional.has_value());
+  auto resources = resources_optional.value();
   ASSERT_EQ(resources.create_maps(*bpftrace, true), 0);
   bpftrace->resources = resources;
 
-  ast::CodegenLLVM codegen(driver.root_, *bpftrace);
+  ast::CodegenLLVM codegen(driver.root.get(), *bpftrace);
   codegen.compile();
 
   ASSERT_EQ(std::distance(bpftrace->maps.begin(), bpftrace->maps.end()), 6);
@@ -57,34 +60,39 @@ TEST(codegen, call_ustack_modes_mapids)
   auto bpftrace = get_mock_bpftrace();
   Driver driver(*bpftrace);
 
-  ASSERT_EQ(driver.parse_str("kprobe:f { @x = ustack(perf); @y = "
-                             "ustack(bpftrace); @z = ustack() }"),
+  ASSERT_EQ(driver.parse_str(
+                "kprobe:f { @w = ustack(raw); @x = ustack(perf); @y = "
+                "ustack(bpftrace); @z = ustack() }"),
             0);
 
   ClangParser clang;
-  clang.parse(driver.root_, *bpftrace);
+  clang.parse(driver.root.get(), *bpftrace);
 
   // Override to mockbpffeature.
   bpftrace->feature_ = std::make_unique<MockBPFfeature>(true);
-  ast::SemanticAnalyser semantics(driver.root_, *bpftrace);
+  ast::SemanticAnalyser semantics(driver.root.get(), *bpftrace);
   ASSERT_EQ(semantics.analyse(), 0);
 
-  ast::ResourceAnalyser resource_analyser(driver.root_);
-  auto resources = resource_analyser.analyse();
+  ast::ResourceAnalyser resource_analyser(driver.root.get());
+  auto resources_optional = resource_analyser.analyse();
+  ASSERT_TRUE(resources_optional.has_value());
+  auto resources = resources_optional.value();
   ASSERT_EQ(resources.create_maps(*bpftrace, true), 0);
   bpftrace->resources = resources;
 
-  ast::CodegenLLVM codegen(driver.root_, *bpftrace);
+  ast::CodegenLLVM codegen(driver.root.get(), *bpftrace);
   codegen.compile();
 
-  ASSERT_EQ(std::distance(bpftrace->maps.begin(), bpftrace->maps.end()), 6);
-  ASSERT_EQ(bpftrace->maps.CountStackTypes(), 2U);
+  ASSERT_EQ(std::distance(bpftrace->maps.begin(), bpftrace->maps.end()), 8);
+  ASSERT_EQ(bpftrace->maps.CountStackTypes(), 3U);
 
   StackType stack_type;
   stack_type.mode = StackMode::perf;
   ASSERT_TRUE(bpftrace->maps.Has(stack_type));
   stack_type.mode = StackMode::bpftrace;
   ASSERT_TRUE(bpftrace->maps.Has(stack_type));
+  stack_type.mode = StackMode::raw;
+  ASSERT_TRUE(bpftrace->maps.Has(stack_type));
 }
 
 } // namespace codegen
