@@ -23,6 +23,8 @@ public:
 #endif
   MOCK_CONST_METHOD1(get_symbols_from_file,
                      std::unique_ptr<std::istream>(const std::string &path));
+  MOCK_CONST_METHOD0(get_symbols_from_traceable_funcs,
+                     std::unique_ptr<std::istream>(void));
   MOCK_CONST_METHOD2(get_symbols_from_usdt,
                      std::unique_ptr<std::istream>(int pid,
                                                    const std::string &target));
@@ -103,9 +105,10 @@ public:
     has_send_signal_ = std::make_optional<bool>(has_features);
     has_get_current_cgroup_id_ = std::make_optional<bool>(has_features);
     has_override_return_ = std::make_optional<bool>(has_features);
-    prog_kfunc_ = std::make_optional<bool>(has_features);
+    has_prog_kfunc_ = std::make_optional<bool>(has_features);
     prog_iter_task_ = std::make_optional<bool>(has_features);
     prog_iter_task_file_ = std::make_optional<bool>(has_features);
+    prog_iter_task_vma_ = std::make_optional<bool>(has_features);
     has_loop_ = std::make_optional<bool>(has_features);
     has_probe_read_kernel_ = std::make_optional<bool>(has_features);
     has_features_ = has_features;
