@@ -5,15 +5,14 @@ import signal
 import sys
 import os
 import time
-from os import environ, uname, devnull
 from distutils.version import LooseVersion
 import re
 from functools import lru_cache
 
 import cmake_vars
 
-BPF_PATH = environ["BPFTRACE_RUNTIME_TEST_EXECUTABLE"]
-ENV_PATH = environ["PATH"]
+BPF_PATH = os.environ["BPFTRACE_RUNTIME_TEST_EXECUTABLE"]
+ENV_PATH = os.environ["PATH"]
 ATTACH_TIMEOUT = 5
 DEFAULT_TIMEOUT = 5
 
@@ -132,6 +131,7 @@ class Runner(object):
         bpffeature["signal"] = output.find("send_signal: yes") != -1
         bpffeature["iter:task"] = output.find("iter:task: yes") != -1
         bpffeature["iter:task_file"] = output.find("iter:task_file: yes") != -1
+        bpffeature["iter:task_vma"] = output.find("iter:task_vma: yes") != -1
         bpffeature["libpath_resolv"] = output.find("bcc library path resolution: yes") != -1
         bpffeature["dwarf"] = output.find("libdw (DWARF support): yes") != -1
         bpffeature["kprobe_multi"] = output.find("kprobe_multi: yes") != -1
@@ -141,7 +141,7 @@ class Runner(object):
 
     @staticmethod
     def run_test(test):
-        current_kernel = LooseVersion(uname()[2])
+        current_kernel = LooseVersion(os.uname()[2])
         if test.kernel_min and LooseVersion(test.kernel_min) > current_kernel:
             print(warn("[   SKIP   ] ") + "%s.%s" % (test.suite, test.name))
             return Runner.SKIP_KERNEL_VERSION_MIN
@@ -167,7 +167,7 @@ class Runner(object):
             print(ok("[ RUN      ] ") + "%s.%s" % (test.suite, test.name))
             if test.requirement:
                 for req in test.requirement:
-                    with open(devnull, 'w') as dn:
+                    with open(os.devnull, 'w') as dn:
                         if subprocess.call(
                             req,
                             shell=True,
@@ -198,7 +198,7 @@ class Runner(object):
             if test.before:
                 before = subprocess.Popen(test.before, shell=True, preexec_fn=os.setsid)
                 waited=0
-                with open(devnull, 'w') as dn:
+                with open(os.devnull, 'w') as dn:
                     # This might not work for complicated cases, such as if
                     # a test program needs to accept arguments. It covers the
                     # current simple calls with no arguments
