@@ -502,6 +502,7 @@ TEST(semantic_analyser, call_print_non_map)
   test(R"_(BEGIN { print((1, 2, "tuple")) })_", 0);
   test(R"_(BEGIN { $x = 1; print($x) })_", 0);
   test(R"_(BEGIN { $x = 1; $y = $x + 3; print($y) })_", 0);
+  test(R"_(BEGIN { print((int8 *)0) })_", 0);
 
   test(R"_(BEGIN { print(3, 5) })_", 1);
   test(R"_(BEGIN { print(3, 5, 2) })_", 1);
@@ -509,7 +510,6 @@ TEST(semantic_analyser, call_print_non_map)
   test(R"_(BEGIN { print(exit()) })_", 10);
   test(R"_(BEGIN { print(count()) })_", 1);
   test(R"_(BEGIN { print(ctx) })_", 1);
-  test(R"_(BEGIN { print((int8 *)0) })_", 10);
 }
 
 TEST(semantic_analyser, call_clear)
@@ -767,7 +767,7 @@ TEST(semantic_analyser, call_uaddr)
 
 TEST(semantic_analyser, call_cgroupid)
 {
-  // Handle args above STRING_SIZE
+  // Handle args above default max-string length (64)
   test("kprobe:f { cgroupid("
        //          1         2         3         4         5         6
        "\"123456789/123456789/123456789/123456789/123456789/123456789/12345\""
@@ -825,6 +825,8 @@ TEST(semantic_analyser, call_stack)
   test("kprobe:f { ustack(3) }", 0);
   test("kprobe:f { kstack(perf, 3) }", 0);
   test("kprobe:f { ustack(perf, 3) }", 0);
+  test("kprobe:f { kstack(raw, 3) }", 0);
+  test("kprobe:f { ustack(raw, 3) }", 0);
 
   // Wrong arguments
   test("kprobe:f { kstack(3, perf) }", 1);
@@ -1027,6 +1029,12 @@ TEST(semantic_analyser, array_access) {
        "struct MyStruct { int y[4]; } "
        "kprobe:f { $s = ((struct MyStruct *)arg0)->y[$2]; }",
        10);
+
+  test(bpftrace,
+       "struct MyStruct { int x; int y[]; } "
+       "kprobe:f { $s = (struct MyStruct *) "
+       "arg0; @y = $s->y[0];}",
+       0);
 }
 
 TEST(semantic_analyser, array_in_map)
@@ -1460,6 +1468,12 @@ TEST(semantic_analyser, tracepoint)
   test("tracepoint:category:event { 1 }", 0);
 }
 
+TEST(semantic_analyser, rawtracepoint)
+{
+  test("rawtracepoint:event { 1 }", 0);
+  test("rawtracepoint:event { args }", 1);
+}
+
 #if defined(ARCH_X86_64) || defined(ARCH_AARCH64)
 TEST(semantic_analyser, watchpoint_invalid_modes)
 {
@@ -1959,9 +1973,6 @@ TEST(semantic_analyser, int_cast_types)
   test("kretprobe:f { @ = (uint16)retval }", 0);
   test("kretprobe:f { @ = (uint32)retval }", 0);
   test("kretprobe:f { @ = (uint64)retval }", 0);
-
-  test("kretprobe:f { @ = (int2)retval }", 1);
-  test("kretprobe:f { @ = (uint2)retval }", 1);
 }
 
 TEST(semantic_analyser, int_cast_usage)
@@ -1985,9 +1996,6 @@ TEST(semantic_analyser, intptr_cast_types)
   test("kretprobe:f { @ = *(uint16*)retval }", 0);
   test("kretprobe:f { @ = *(uint32*)retval }", 0);
   test("kretprobe:f { @ = *(uint64*)retval }", 0);
-
-  test("kretprobe:f { @ = *(int2*)retval }", 1);
-  test("kretprobe:f { @ = *(uint2*)retval }", 1);
 }
 
 TEST(semantic_analyser, intptr_cast_usage)
@@ -2119,9 +2127,27 @@ TEST(semantic_analyser, unwatch)
 TEST(semantic_analyser, struct_member_keywords)
 {
   std::string keywords[] = {
-    "arg0", "args", "curtask", "func", "gid" "rand", "uid",
-    "avg", "cat", "exit", "kaddr", "min", "printf", "usym",
-    "kstack", "ustack", "bpftrace", "perf", "uprobe", "kprobe",
+    "arg0",
+    "args",
+    "curtask",
+    "func",
+    "gid"
+    "rand",
+    "uid",
+    "avg",
+    "cat",
+    "exit",
+    "kaddr",
+    "min",
+    "printf",
+    "usym",
+    "kstack",
+    "ustack",
+    "bpftrace",
+    "perf",
+    "raw",
+    "uprobe",
+    "kprobe",
   };
   for(auto kw : keywords)
   {
@@ -2434,8 +2460,6 @@ TEST(semantic_analyser, tuple)
   test(R"_(BEGIN { @t = (1, 2); @t = (4, "other"); })_", 10);
   test(R"_(BEGIN { @t = (1, 2); @t = 5; })_", 1);
   test(R"_(BEGIN { @t = (1, count()) })_", 1);
-  test(R"_(BEGIN { @t = (1, (aaa)0) })_", 1);
-  test(R"_(BEGIN { @t = (1, !(aaa)0) })_", 1);
 }
 
 TEST(semantic_analyser, tuple_indexing)
@@ -2565,10 +2589,56 @@ TEST(semantic_analyser, call_path)
   test("END { $k = path( 1 ) }", 1);
 }
 
+TEST(semantic_analyser, call_offsetof)
+{
+  test("struct Foo { int x; long l; char c; } \
+        BEGIN { @x = offsetof(struct Foo, x); }",
+       0);
+  test("struct Foo { int comm; } \
+        BEGIN { @x = offsetof(struct Foo, comm); }",
+       0);
+  test("struct Foo { int ctx; } \
+        BEGIN { @x = offsetof(struct Foo, ctx); }",
+       0);
+  test("struct Foo { int args; } \
+        BEGIN { @x = offsetof(struct Foo, args); }",
+       0);
+  test("struct Foo { int x; long l; char c; } \
+        struct Bar { struct Foo foo; int x; } \
+        BEGIN { @x = offsetof(struct Bar, x); }",
+       0);
+  test("struct Foo { int x; long l; char c; } \
+        union Bar { struct Foo foo; int x; } \
+        BEGIN { @x = offsetof(union Bar, x); }",
+       0);
+  test("struct Foo { int x; long l; char c; } \
+        struct Fun { struct Foo foo; int (*call)(void); } \
+        BEGIN { @x = offsetof(struct Fun, call); }",
+       0);
+  test("struct Foo { int x; long l; char c; } \
+        BEGIN { $foo = (struct Foo *)0; \
+        @x = offsetof(*$foo, x); }",
+       0);
+  test("struct Foo { int x; long l; char c; } \
+        struct Ano { \
+          struct { \
+            struct Foo foo; \
+            int a; \
+          }; \
+          long l; \
+        } \
+        BEGIN { @x = offsetof(struct Ano, a); }",
+       0);
+  test("struct Foo { int x; long l; char c; } \
+        BEGIN { @x = offsetof(struct Foo, __notexistfield__); }",
+       1);
+  test("BEGIN { @x = offsetof(__passident__, x); }", 1);
+  test("BEGIN { @x = offsetof(struct __notexiststruct__, x); }", 1);
+}
+
 TEST(semantic_analyser, int_ident)
 {
   test("BEGIN { sizeof(int32) }", 0);
-  test("BEGIN { print(int32) }", 1);
 }
 
 TEST(semantic_analyser, tracepoint_common_field)
@@ -2682,13 +2752,18 @@ TEST_F(semantic_analyser_btf, iter)
 {
   test("iter:task { 1 }", 0);
   test("iter:task_file { 1 }", 0);
+  test("iter:task_vma { 1 }", 0);
   test("iter:task { $x = ctx->task->pid }", 0);
   test("iter:task_file { $x = ctx->file->ino }", 0);
+  test("iter:task_vma { $x = ctx->vma->vm_start }", 0);
   test("iter:task { $x = args->foo; }", 1);
   test("iter:task_file { $x = args->foo; }", 1);
+  test("iter:task_vma { $x = args->foo; }", 1);
   test("iter:task { printf(\"%d\", ctx->task->pid); }", 0);
   test("iter:task_file { printf(\"%d\", ctx->file->ino); }", 0);
+  test("iter:task_vma { printf(\"%lx\", ctx->vma->vm_start); }", 0);
   test("iter:task,iter:task_file { 1 }", 1);
+  test("iter:task,iter:task_vma { 1 }", 1);
   test("iter:task,f:func_1 { 1 }", 1);
 }
 
