@@ -46,6 +46,24 @@ TEST(utils, split_string)
   EXPECT_EQ(split_string("foo-bar", '-'), tokens_foo_bar);
 }
 
+TEST(utils, split_addrrange_symbol_module)
+{
+  std::tuple<std::string, std::string, std::string> tokens_ar_sym = {
+    "0xffffffff85201511-0xffffffff8520152f", "first_nmi", ""
+  };
+  std::tuple<std::string, std::string, std::string> tokens_ar_sym_mod = {
+    "0xffffffffc17e9373-0xffffffffc17e94ff", "vmx_vmexit", "kvm_intel"
+  };
+
+  EXPECT_EQ(split_addrrange_symbol_module(
+                "0xffffffff85201511-0xffffffff8520152f	first_nmi"),
+            tokens_ar_sym);
+  EXPECT_EQ(split_addrrange_symbol_module(
+                "0xffffffffc17e9373-0xffffffffc17e94ff	vmx_vmexit "
+                "[kvm_intel]"),
+            tokens_ar_sym_mod);
+}
+
 TEST(utils, wildcard_match)
 {
   std::vector<std::string> tokens_not = {"not"};
@@ -222,6 +240,28 @@ TEST(utils, get_cgroup_path_in_hierarchy)
   }
 }
 
+TEST(utils, parse_kconfig)
+{
+  char path[] = "/tmp/configXXXXXX";
+  int fd = mkstemp(path);
+  const std::string config = "# Intro comment\n"
+                             "CONFIG_YES=y\n"
+                             "CONFIG_MOD=m\n"
+                             "CONFIG_VAL=42\n"
+                             "# CONFIG_NO is not set";
+  EXPECT_EQ(write(fd, config.c_str(), config.length()), config.length());
+  setenv("BPFTRACE_KCONFIG_TEST", path, true);
+  close(fd);
+
+  KConfig kconfig;
+  ASSERT_TRUE(kconfig.has_value("CONFIG_YES", "y"));
+  ASSERT_TRUE(kconfig.has_value("CONFIG_MOD", "m"));
+  ASSERT_TRUE(kconfig.has_value("CONFIG_VAL", "42"));
+  ASSERT_EQ(kconfig.config.find("CONFIG_NO"), kconfig.config.end());
+
+  unlink(path);
+}
+
 } // namespace utils
 } // namespace test
 } // namespace bpftrace
